/**
  *  \file client/chart/movemarkermode.cc
  *  \brief Starchart Mode: WMoveMarkerChartMode
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/chart/movemarkermode.h"
#include "client/chartloc.h"
#include "gfx/gfxcontext.h"
#include "gfx/palette.h"
#include "ui/event.h"
#include "ui/uiglobal.h"
#include "util/nls.h"
#include "game/global.h"

WMoveMarkerChartMode::WMoveMarkerChartMode(GDrawingContainer::iterator drawing)
    : WMarkerChartMode(drawing)
{ }

void
WMoveMarkerChartMode::doChartLock(GChartLocation& loc, TLockMode lm, bool marked)
{
    GPoint pt = loc.getLocation();
    GLockData locker(pt, marked);
    if (*drawing) {
        locker.setIgnore(*drawing);
    }
    findItem(getDisplayedTurn().getCurrentUniverse(), locker, lm);
    if (const GMapObject* obj = locker.getObject()) {
        loc.setCurrentObject(*obj);
    }
    loc.setLocation(locker.getPoint().getSimpleNearestAlias(loc.getLocation()));
}

void
WMoveMarkerChartMode::onMove(GChartLocation& loc)
{
    if (GDrawing* d = *drawing) {
        if (d->getPos() != loc.getLocation()) {
            d->setPos(loc.getLocation());
            loc.markChanged();
        }
    }
    WMarkerChartMode::onMove(loc);
}

void
WMoveMarkerChartMode::drawOverlays(GfxCanvas& can, GChartLocation& loc)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(GfxStandardColorScheme::instance);
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setColor(COLOR_WHITE);

    GfxPoint sc = loc.getViewport().getCenter();

    string_t text = _(" Move marker, end with [ESC]. ");
    int wi = ctx.getFont().getTextWidth(text);
    int x = sc.x - wi / 2;
    int y = sc.y + 20;
    drawSolidBar(ctx, GfxRect(x, y, wi, font_heights[FONT_NORMAL]), COLOR_RED);
    outText(ctx, x, y, text);

    WMarkerChartMode::drawOverlays(can, loc);
}

bool
WMoveMarkerChartMode::handleEvent(const UIEvent& event, GChartLocation& loc)
{
    if (event.type == ev_Key) {
        switch (event.key.code) {
         case 'v':
            // Exit move mode
            delete this;
            return true;
         case SDLK_F9:
         case SDLK_F9 + ss_Alt:
            // Change comment
            if (*drawing) {
                editMarkerComment(**drawing);
                loc.markChanged();
            }
            return true;

         case SDLK_RETURN:
            // Locking; mostly copied from standardmode
            doChartLock(loc, lm_Right, false);
            return true;
         case SDLK_SPACE:
            doChartLock(loc, lm_Left, false);
            return true;
         case ss_Ctrl + SDLK_RETURN:
            doChartLock(loc, lm_Right, true);
            return true;
         case ss_Ctrl + SDLK_SPACE:
            doChartLock(loc, lm_Left, true);
            return true;
        }
    }
    return WMarkerChartMode::handleEvent(event, loc);
}
