/**
  *  \file client/widgets/pluginlist.cc
  *  \brief Class WPluginList
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/pluginlist.h"
#include "util/nls.h"
#include "cpluslib/format.h"

/** Constructor. */
WPluginList::WPluginList(int id)
    : UIListbox(id, 10, 20*font_ems[FONT_NORMAL], 0)
{ }

WPluginList::~WPluginList()
{ }

/** Initialize.
    \param mgr PluginManager to display */
void
WPluginList::init(const PluginManager& mgr)
{
    string_t current = getCurrentPluginId();

    std::vector<Plugin*> plugs;
    mgr.enumPlugins(plugs, false);

    content.clear();
    int newPos = getCurrent();
    for (size_t i = 0; i < plugs.size(); ++i) {
        const Plugin& plug = *plugs[i];
        content.push_back(Item(plug.getId(), plug.getName(), plug.isLoaded()));
        if (plug.getId() == current) {
            newPos = i;
        }
    }
    setItemHeight(font_heights[FONT_NORMAL_BOLD] + font_heights[FONT_SMALL] + 4);
    setSize(content.size());
    scrollTo(newPos, 1);
    adjustPosition();
    drawWidget();
    sig_moved.raise(getId());
}

void
WPluginList::drawPart(GfxCanvas& can, int from, int to)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    if (content.empty()) {
        GfxRect area = getExtent();
        drawBackground(ctx, area);
        ctx.setColor(UIColor::tc_Static);
        ctx.setTextAlign(1, 0);
        ctx.setTransparentBackground();
        area.consumeY(10);
        outTextF(ctx, area, _("No plugins installed."));
    } else {
        ctx.setTextAlign(0, 0);
        for (int i = from; i < to; ++i) {
            // Prepare
            GfxRect area;
            prepareEntry(ctx, ColorStyle, i, &area);

            // Draw
            int n = getTop() + i;
            if (n >= 0 && n < int(content.size())) {
                area.consumeX(5);
                area.consumeY(2);
                ctx.useFont(*fonts[FONT_NORMAL_BOLD]);
                outTextF(ctx, area.splitY(font_heights[FONT_NORMAL]), content[n].name);
                ctx.useFont(*fonts[FONT_SMALL]);
                ctx.setColor(content[n].loaded ? UIColor::tc_Faded : UIColor::tc_Red);
                outTextF(ctx, area, format("(%d, %d)",
                                           content[n].id,
                                           content[n].loaded ? _("loaded") : _("not loaded")));
            }
        }
    }
}

/** Get Id of currently-selected plugin.
    If none is selected, returns an empty string. */
string_t
WPluginList::getCurrentPluginId() const
{
    int p = getCurrent();
    if (p < int(content.size())) {
        return content[p].id;
    } else {
        return string_t();
    }
}
