/**
  *  \file flak/flak-db.h
  *  \brief FLAK VCR Database Implementation
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_FLAK_FLAK_DB_H
#define PCC_V2_FLAK_FLAK_DB_H

#include "flak/flak.h"
#include "flak/flakdefs.h"
#include "cpluslib/buffer.h"
#include "game/vcrdb.h"
#include "cpluslib/ptrvector.h"
#include "game/vcrobj.h"
#include "game/timestamp.h"

class Stream;

class FlakVcrEntry : public GVcrEntry {
    FlakBattle    battle;
    TFlakBattle   header;

    ptr_vector<GVcrObject> before;
    ptr_vector<GVcrObject> after;
    bool have_after;

 public:
    FlakVcrEntry(string_t name, const Buffer<char>& in);
    FlakVcrEntry(const FlakBattle& the_battle);
    ~FlakVcrEntry();

    int getPositionX() const
        { return header.x; }
    int getPositionY() const
        { return header.y; }
    int32 getTotalTime() const
        { return header.total_time; }

    const FlakBattle& getBattle() const
        { return battle; }
    FlakBattle& getBattle()
        { return battle; }

    // GVcrEntry:
    int getNumObjects() const;
    GVcrObject& getObject(int slot, bool after);
    int getOutcome(int slot);
    void prepareResult(int resultLevel, ProgressMonitor& pm);
    Playability getPlayability();
    string_t getAlgorithmName(bool translate) const;
    bool isESBActive();

 private:
    void initBefore();
};

class GFlakVcrDatabase : public GVcrDatabase {
    GFlakVcrDatabase(const GFlakVcrDatabase&);
    GFlakVcrDatabase& operator=(const GFlakVcrDatabase&);

    ptr_vector<FlakVcrEntry> battles;

    TFlakHeader last_header;

    void readOneBattle(Stream& s);
    void loadFromFile(Stream& s);
 public:
    GFlakVcrDatabase();
    GFlakVcrDatabase(Stream& s);
    ~GFlakVcrDatabase();

    void addBattle(const FlakBattle& b);
    GTimestamp getTimestamp();

    int getNumBattles() const;
    FlakVcrEntry& getBattle(int nr);
};

#endif
