/**
  *  \file game/actions/baseammo.h
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_ACTIONS_BASEAMMO_H
#define PCC_V2_GAME_ACTIONS_BASEAMMO_H

#include "game/actions/cargocost.h"
#include "game/cargotype.h"
#include "game/planet.h"
#include "game/planettransfer.h"

class GCargoContainer;
class GPlanet;

/** Ammunition building. This transaction allows to build ammunition
    from a planet, and place it onto any receiver object. */
class GStarbaseBuildAmmoAction : public GCargoCostTransaction {
 public:
    enum {
        /// Minimum index for accessor functions.
        min_index = 1,
        /// Maximum index for accessor functions.
        max_index = NUM_TORPS+1
    };
 private:
    /// Planet who will pay for new ammo.
    GPlanetTransfer& pay;
    /// Receiver of new ammo.
    GCargoContainer& receiver;
    /// Amount of ammo added.
    int32_t added[max_index];

 protected:
    void   commit();
    void   cancel();

 public:
    GStarbaseBuildAmmoAction(GPlanetTransfer& pay, GCargoContainer& receiver);
    ~GStarbaseBuildAmmoAction();

    bool    canHave(int index) const;
    int32_t getAmount(int index) const;
    void    setAmount(int index, int32_t amount);
    int32_t limitAmount(int index, int32_t amount) const;
    int     getTechUpgradeNeeded(int index) const;

    GPlanet& getPlanet()
        { return pay.getPlanet(); }

    static GCargoType getCargoTypeFromIndex(int index);
};

#endif
