/**
  *  \file game/actions/planetsupplies.h
  *  \brief Planetary Action: Supply Sale
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_ACTIONS_PLANETSUPPLIES_H
#define PCC_V2_GAME_ACTIONS_PLANETSUPPLIES_H

#include "game/planet.h"
#include "game/transact.h"
#include "game/planettransfer.h"

/*! \class GPlanetSellSuppliesAction
    \brief Planetary Action: Supply Sale.

    This transaction allows you to sell supplies.

    FIXME: this should also allow buying supplies. */
class GPlanetSellSuppliesAction : public GTransaction {
    GPlanet&         the_planet;
    GPlanetTransfer  transfer;
    int32_t          amount;
    int32_t          reserved_sup;
 public:
    GPlanetSellSuppliesAction(GPlanet& the_planet);
    ~GPlanetSellSuppliesAction();

    int32_t getMaximumAmount() const;
    void    setAmount(int32_t amount);
    int32_t getAmount() const;

    Signal0 sig_changed;
 protected:
    void commit();
    void cancel();
};

inline int32_t
GPlanetSellSuppliesAction::getAmount() const
{
    return amount;
}

#endif
