/**
  *  \file game/planetform.h
  *  \brief Planet Formulas
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_PLANETFORM_H
#define PCC_V2_GAME_PLANETFORM_H

#include "game/planet.h"

/* Structure formulas */
mp32_t  getMaxStructures(const GPlanet& pl, TPlanetaryStructure ps);
int     getMaxStructures(const GPlanet& pl, TPlanetaryStructure ps, int32_t clans);

/* Colonist formulas */
mn32_t  getColonistChange(const GPlanet& pl, int tax, int mifa);
mn32_t  getColonistChange(const GPlanet& pl);
int32_t getColonistDue(const GPlanet& pl, int tax);
int32_t getColonistDueLimited(const GPlanet& pl, int tax, int32_t& rem_inc);
mp16_t  getColonistSafeTax(const GPlanet& pl, int mifa);
mp32_t  getMaxSupportedColonists(const GPlanet& pl, int player);
mp32_t  getMaxSupportedColonists(const GPlanet& pl);

/* Native formulas */
mn32_t  getNativeChange(const GPlanet& pl);
mn32_t  getNativeChange(const GPlanet& pl, int tax, int mifa);
int32_t getNativeDue(const GPlanet& pl, int tax);
int32_t getNativeDue(int tax, int race, int gov, int32_t pop, int owner);
int32_t getNativeDueLimited(const GPlanet& pl, int tax, int32_t rem_inc);
mp16_t  getNativeSafeTax(const GPlanet& pl,int mifa);
mp16_t  getNativeBaseTax(const GPlanet& pl, int happyTarget);
int32_t getBovinoidSupplyContribution(const GPlanet& pl);
int32_t getBovinoidSupplyContribution(int32_t pop, int owner);
int32_t getBovinoidSupplyContributionLimited(const GPlanet& pl);
int32_t getAmorphousBreakfast(int happy);

/* Mining formulas */
int     getMiningCapacity(const GPlanet& pl, int index, int mines);

/* Starbase formulas */
int32_t getBaseTechCost(int player, int fromTech, int toTech);

#endif
