/**
  *  \file reshack/charname.h
  *  \brief Resource Editor: Character Names
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_RESHACK_CHARNAME_H
#define PCC_V2_RESHACK_CHARNAME_H

#include <vector>
#include "cpluslib/string.h"
#include "cpluslib/types.h"
#include "ui/ui.h"
#include "ui/listbox.h"

/** List of characters. */
typedef std::vector<uint32_t> CharacterList;

void     loadCharacterNames();
string_t getCharacterName(uint32_t ch);
void     matchCharacterNames(string_t expr, uint32_t limit, CharacterList& out);

/** Character generator. */
class RHCharacterGenerator {
 public:
    virtual ~RHCharacterGenerator();
    virtual bool check(uint32_t chars[], uint32_t nchars) = 0;

    bool generate(uint32_t id);
};

/** Character name widget. Displays a single character's name. */
class RHCharacterName : public UIWidget {
    uint32_t character;
 public:
    RHCharacterName();

    void getLayoutInfo(LayoutInfo& info);
    void drawContent(GfxCanvas& can);
    void setCharacter(uint32_t character);
};

/** Character name list. Displays a list of character names. */
class RHCharacterNameList : public UIListbox {
    CharacterList& charlist;
 public:
    RHCharacterNameList(int id, CharacterList& charlist);

    void drawPart(GfxCanvas& can, int from, int to);
    void update();
};

#endif
