#!/usr/bin/perl -w
#
#  PCC2 Object File Size Display
#

use strict;

foreach (@ARGV) {
    if (/^-/) {
        die "No options accepted.";
    } else {
        size($_);
    }
}

sub size {
    my $fn = shift;
    my $result;
    open FILE, "< $fn" or die "$fn: $!\n";
    binmode FILE;

    # Load header
    my $header;
    if (read(FILE, $header, 14) != 14)          { die "File format error" }
    if (substr($header, 0, 6) ne "CCobj\032")   { die "File format error" }
    if (unpack("v",substr($header, 8, 2)) != 4) { die "File format error" }
    $result .= $header;
    my $entryId = unpack("V", substr($header, 10, 4));

    print "Content of $fn:\n\n";
    print "  Code    Literals    Total   Routine\n";

    # Show it
    my $code = 0;
    my $lit = 0;
    my $size = 0;
    my $debug = 0;
    while (my $p = loadObject(\*FILE)) {
        my $c = length($p->{properties}[4][1] or '');
        my $l = length($p->{properties}[2][1] or '');
        my $s = $p->{size};
        printf "%8d  %8d  %8d  %s\n", $c, $l, $s,
          ($p->{properties}[6][1] or ($entryId == $p->{id} ? '(entry)' : '(unnamed)'));
        $code += $c;
        $lit += $l;
        $size += $s;
        $debug += length($p->{properties}[7][1] or '') + length($p->{properties}[8][1] or '');
    }
    printf "%8d  %8d  %8d  %s\n", $code, $lit, $size, "-> Total";
    printf "%d bytes debug information\n", $debug if $debug;
    print "\n";
    close FILE;
}

sub loadObject {
    my $fd = shift;

    # Read header
    my $header;
    my $nbytes = read($fd, $header, 16);
    if ($nbytes == 0) { return undef; }
    if ($nbytes != 16) { die "File format error" }

    # Read content
    my ($typ, $id, $size, $nprop) = unpack "V*", $header;
    if ($size < $nprop*8) { die "File format error" }
    my $origSize = $size;
    # print "Object: type=$typ, id=$id, size=$size, nprop=$nprop\n";

    # Read property headers
    my $phead;
    my @phead;
    if (read($fd, $phead, 8*$nprop) != 8*$nprop) { die "File format error" }
    @phead = unpack "V*", $phead;
    $size -= 8*$nprop;

    # Read properties
    my $props = [];
    for (my $i = 0; $i < $nprop; ++$i) {
        my $count = shift @phead;
        my $psize = shift @phead;
        # print "  Property $i: $count elems, $psize bytes\n";
        if ($psize > $size) { die "File format error" }
        $size -= $psize;
        my $pdata;
        if (read($fd, $pdata, $psize) != $psize) { die "File format error" }
        push @$props, [$count,$pdata];
    }

    if ($size != 0) { die "File format error" }

    return { id => $id,
             type => $typ,
             properties => $props,
             size => $origSize };
}
