/**
  *  \file u/t_util_bsearch.cc
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "util/bsearch.h"
#include "u/t_util.h"

#include <string.h>             // strcmp

using namespace std;            // strcmp

static const char*const text[] = {
    "Apple",
    "Bullfrog",
    "Compaq",
    "Dell",
    "Ensoniq",
    "Gateway",
    "Honeywell",
    "IBM",
};

namespace {
    struct stringcomp {
        const char* searchstr;
        bool isLess(const char* p) const { return strcmp(searchstr, p) < 0; }
        bool isEqual(const char* p) const { return strcmp(searchstr, p) == 0; }
        stringcomp(const char* s) : searchstr(s) { }
    };
}

void
UtilBSearchTestSuite::testBSearch()
{
    TS_ASSERT_EQUALS(bin_search(text, 8, stringcomp("Dell")), 3);
    TS_ASSERT_EQUALS(bin_search(text, 8, stringcomp("00")), -1);
    TS_ASSERT_EQUALS(bin_search(text, 8, stringcomp("Zilog")), -1);

    for (int i = 0; i < 8; ++i)
        TS_ASSERT_EQUALS(bin_search(text, 8, stringcomp(text[i])), i);
    for (int i = 0; i < 7; ++i)
        TS_ASSERT_EQUALS(bin_search(text, 7, stringcomp(text[i])), i);
    TS_ASSERT_EQUALS(bin_search(text, 7, stringcomp("IBM")), -1);
    TS_ASSERT_EQUALS(bin_search(text, 8, stringcomp("HP")), -1);
    TS_ASSERT_EQUALS(bin_search(text, 8, stringcomp("Creative")), -1);
    TS_ASSERT_EQUALS(bin_search(text, 1, stringcomp("Apple")), 0);
    TS_ASSERT_EQUALS(bin_search(text, 0, stringcomp("Apple")), -1);

    /* same again on sequences */
    TS_ASSERT_EQUALS(bin_search_seq(text, text+8, stringcomp("Dell")), text+3);
    TS_ASSERT_EQUALS(bin_search_seq(text, text+8, stringcomp("00")), text+8);
    TS_ASSERT_EQUALS(bin_search_seq(text, text+8, stringcomp("Zilog")), text+8);
    for (int i = 0; i < 8; ++i)
        TS_ASSERT_EQUALS(bin_search_seq(text, text+8, stringcomp(text[i])), text+i);
    TS_ASSERT_EQUALS(bin_search_seq(text, text+1, stringcomp("Apple")), text);
    TS_ASSERT_EQUALS(bin_search_seq(text, text, stringcomp("Apple")), text);
}
