/**
  *  \file ui/richparser.h
  *  \brief Rich Text Parsing
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_RICHPARSER_H
#define PCC_V2_UI_RICHPARSER_H

#include "util/xmlread.h"
#include "ui/richtext.h"

/** Rich text parser, basic version.
    This implements parsing of flow-text markup from XML. */
class RichParser {
 public:
    RichParser(XmlReader& rdr);

    // Utilities
    void next();
    bool isOpen(const char* what);
    void parseUnknown();

    // Parser
    RichText parseText(bool keepFormat);
    RichText parseTextItem(bool keepFormat);

    RichText parse();

    static void appendText(RichText& out, bool& haveSpace, const string_t& in);
    static RichText renderKeys(const string_t& name);

    static RichText textFromXml(string_t source);

 protected:
    XmlReader& rdr;
    XmlReader::Token current;
};


/** Constructor. */
inline
RichParser::RichParser(XmlReader& rdr)
    : rdr(rdr),
      current(rdr.Eof)
{ }

/** Advance to next tag. */
inline void
RichParser::next()
{
    current = rdr.read();
}

#endif
