/**
  *  \file client/screens/taskscreen.h
  *  \brief Auto Task Screen
  *
  *  (c) 2001-2016 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_SCREENS_TASKSCREEN_H
#define PCC_V2_CLIENT_SCREENS_TASKSCREEN_H

#include "int/autotask.h"
#include "game/objtypes.h"
#include "client/screens.h"
#include "client/cscreen.h"
#include "game/objsel.h"

class IntNotificationMessage;

/** Object type that tracks an Auto Task. In addition to an underlying
    object type, this tracks a decompiled auto task and a cursor.
    Descendants will supply a getTask() routine to access the actual
    auto tasks.

    Before this can be used, you must call init() to load the first
    task; after using, call done() to save the last one. */
class WAutoTaskObjectSelection : public GProxyObjectSelection,
                                 private GObjectSelectionObserver
{
    IntAutoTaskEditor editor;
    IntExecutionContext* current_process;
    uint32_t cursor;

 public:
    WAutoTaskObjectSelection(GObjectSelection& selection);
    ~WAutoTaskObjectSelection();

    void init();
    void done();

    void setCursor(uint32_t pos);
    uint32_t getCursor() const;
    IntNotificationMessage* getNotificationMessage() const;

    const IntAutoTaskEditor& getEditor() const;

    void replace(uint32_t pos, uint32_t nold, const string_t& line);
    void replace(uint32_t pos, uint32_t nold, const string_t* line, uint32_t nnew);
    void insert(const string_t& line);
    void setPC();

    void createCommonCommand(UIBaseWidget& sender, bool allowName);

    Signal1<bool> sig_task_changed;

    using GProxyObjectSelection::getCurrentObject;
    using GProxyObjectSelection::getCurrentUniverse;

 protected:
    /** Get this object's task. When create=false, return 0 if there isn't any.
        When create=true, always create it and never return 0. */
    virtual IntExecutionContext* getTask(bool create) = 0;

 private:
    void onCurrentChanged();
    void onObjectChanged();
    void loadTask(IntExecutionContext* process);
};


/** Auto Task screen. Implements basic auto task functionality. */
class WAutoTaskScreen : public WControlScreen {
 protected:
    WAutoTaskObjectSelection& selection;
 public:
    WAutoTaskScreen(WAutoTaskObjectSelection& selection, WScreenId::ScreenId screenId);
    ~WAutoTaskScreen();

    bool handleEvent(const UIEvent& event, bool second_pass);

    void handleSave();
    void handleLoad();
};

#endif
